"use strict";

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _crypto = require("crypto");
var _fs = require("fs");
var _path = require("path");

var async = require("neo-async");
var cheerio = require("cheerio");
var ejs = require("ejs");
var merge = require("webpack-merge");
var mkdirp = require("mkdirp");
var tmp = require("tmp");
var touch = require("touch");
var rimraf = require("rimraf");
var React = require("react");
var ReactDOMServer = require("react-dom/server");
var webpack = require("webpack");

var prettyConsole = require("../libs/pretty_console");

var cwd = process.cwd();

module.exports = function writePages(params, finalCb) {
  async.each(params.pages, function (d, cb) {
    var page = d.page,
        path = d.path;


    processPage({
      page: page,
      path: path,
      outputPath: params.output,
      templates: params.templates
    }, cb);
  }, finalCb);
};

function processPage(_ref, cb) {
  var _ref$page = _ref.page,
      page = _ref$page === undefined ? "" : _ref$page,
      _ref$outputPath = _ref.outputPath,
      outputPath = _ref$outputPath === undefined ? "" : _ref$outputPath,
      _ref$path = _ref.path,
      path = _ref$path === undefined ? "" : _ref$path,
      _ref$templates = _ref.templates,
      templates = _ref$templates === undefined ? {} : _ref$templates;

  var renderPage = require(_path.join(outputPath, "site.js")).renderPage;

  renderPage(page, function (err, _ref2) {
    var html = _ref2.html,
        page = _ref2.page,
        context = _ref2.context;

    if (err) {
      return cb(err);
    }

    var $ = cheerio.load(html);
    var components = $(".interactive").map(function (i, el) {
      var $el = $(el);
      var id = $el.attr("id");
      var props = $el.data("props");

      return {
        id: id,
        name: "Interactive" + i,
        path: _path.join(cwd, id),
        props: convertToJS(props)
      };
    }).get();
    var jsFiles = [];

    if (components.length) {
      // XXX: Should this bail early?
      components.forEach(function (component) {
        if (!_fs.existsSync(component.path)) {
          prettyConsole.log("Failed to find", component.path);
        }
      });

      // Calculate hash based on filename and section so we can check whether
      // to generate a bundle at all. Use a relative path so the project directory
      // can be moved around.
      var filename = calculateMd5(_path.relative(cwd, path).split("/").filter(function (a) {
        return a;
      }).slice(0, -1).join("/") + components.map(function (c) {
        return c.id + "=" + c.props;
      }).join(""));
      var interactivePath = _path.join(outputPath, filename + ".js");

      // Attach generated file to template
      jsFiles.push("/" + filename + ".js");

      // If the bundle exists already, skip generating
      if (!_fs.existsSync(interactivePath)) {
        var _webpackConfig$entry;

        var interactiveIndexEntry = ejs.compile(templates.interactiveIndex.file)({
          components: components
        });
        var entry = ejs.compile(templates.interactive.file)({
          components: components
        });

        // Touch output so that other processes get a clue
        touch.sync(interactivePath);

        // Write to a temporary files so we can point webpack to that
        var interactiveEntryTmpFile = tmp.fileSync();
        var entryTmpFile = tmp.fileSync();

        // XXX: convert to async
        _fs.writeFileSync(interactiveEntryTmpFile.name, interactiveIndexEntry);
        _fs.writeFileSync(entryTmpFile.name, entry);

        // XXX: should it be possible to tweak this? now we are picking
        // the file by convention
        var webpackConfigPath = _path.join(cwd, "webpack.config.js");
        var interactiveConfig = require(webpackConfigPath)("interactive");
        var webpackConfig = merge(interactiveConfig, {
          resolve: {
            modules: [cwd, _path.join(cwd, "node_modules")],
            alias: generateAliases(components)
          },
          resolveLoader: {
            modules: [cwd, _path.join(cwd, "node_modules")]
          },
          plugins: [new webpack.DefinePlugin({
            __DEV__: false
          })]
        });

        var interactiveIndexEntryName = filename + "-interactive-entry";

        // Override webpack configuration to process correctly
        webpackConfig.entry = (_webpackConfig$entry = {}, _defineProperty(_webpackConfig$entry, interactiveIndexEntryName, interactiveEntryTmpFile.name), _defineProperty(_webpackConfig$entry, filename, entryTmpFile.name), _webpackConfig$entry);

        // Merge output to avoid overriding publicPath
        webpackConfig.output = merge(interactiveConfig.output, {
          filename: "[name].js",
          path: outputPath,
          publicPath: "/",
          libraryTarget: "umd" // Needed for interactive index exports to work
        });

        return webpack(webpackConfig, function (err2, stats) {
          if (err2) {
            return cb(err2);
          }

          if (stats.hasErrors()) {
            return cb(stats.toString("errors-only"));
          }

          var interactiveIndexPath = _path.join(outputPath, interactiveIndexEntryName);
          var interactiveComponents = require(interactiveIndexPath);
          var renderErrors = [];

          // Render initial HTML for each component
          $(".interactive").each(function (i, el) {
            var $el = $(el);
            var props = $el.data("props");

            try {
              $el.html(ReactDOMServer.renderToStaticMarkup(React.createElement(interactiveComponents["Interactive" + i], props)));
            } catch (renderErr) {
              renderErrors.push(renderErr);
            }
          });

          if (renderErrors.length) {
            return cb(renderErrors[0]);
          }

          rimraf.sync(interactiveIndexPath + ".*");

          // Wrote a bundle, compile through ejs now
          var data = ejs.compile(templates.page.file)({
            htmlWebpackPlugin: {
              options: {
                context: _extends({}, context, page.file, templates.page, {
                  jsFiles: [].concat(_toConsumableArray(templates.page.jsFiles), jsFiles)
                })
              }
            },
            webpackConfig: {
              html: $.html()
            }
          });

          return writePage({ path: path, data: data, page: page }, cb);
        });
      }
    }

    // No need to go through webpack so go only through ejs
    var data = ejs.compile(templates.page.file)({
      htmlWebpackPlugin: {
        options: {
          context: _extends({}, page.file, templates.page, {
            jsFiles: [].concat(_toConsumableArray(templates.page.jsFiles), jsFiles)
          })
        }
      },
      webpackConfig: {
        html: html
      }
    });

    return writePage({ path: path, data: data }, cb);
  });
}

function convertToJS(props) {
  var ret = "";

  Object.keys(props).forEach(function (prop) {
    var v = props[prop];

    ret += prop + ": " + JSON.stringify(v) + ",";
  });

  return "{" + ret + "}";
}

function generateAliases(components) {
  var ret = {};

  components.forEach(function (_ref3) {
    var id = _ref3.id,
        path = _ref3.path;

    ret[id] = path;
  });

  return ret;
}

function writePage(_ref4, cb) {
  var path = _ref4.path,
      data = _ref4.data;

  mkdirp(_path.dirname(path), function (err) {
    if (err) {
      return cb(err);
    }

    return _fs.writeFile(path, data, function (err2) {
      if (err2) {
        return cb(err2);
      }

      prettyConsole.log("Finished writing page", path);

      return cb();
    });
  });
}

function calculateMd5(input) {
  return _crypto.createHash("md5").update(input).digest("hex");
}