"use strict";

var path = require("path");
var merge = require("webpack-merge");
var webpack = require("webpack");
var WebpackDevServer = require("webpack-dev-server");
var devConfig = require("../config/dev");

module.exports = function (config) {
  var devConfigParams = {
    entry: {
      main: [
      // As inline mode doesn't work with the Node version (no access to
      // webpack configuration), we have to add entries by hand.
      "webpack-dev-server/client?http://localhost:" + config.antwar.port, "webpack/hot/dev-server", path.join(__dirname, "./entry.js")]
    },
    plugins: [new webpack.HotModuleReplacementPlugin()],
    devtool: "eval"
  };

  return devConfig(config).then(function (c) {
    runServer(config.antwar, merge(c, devConfigParams));
  });
};

function runServer(siteConfig, webpackConfig) {
  var port = siteConfig.port;
  var console = siteConfig.console;

  new WebpackDevServer(webpack(webpackConfig), {
    contentBase: path.join(process.cwd(), "./.antwar/build"),
    hot: true,
    historyApiFallback: true,
    stats: webpackConfig.stats
  }).listen(port, function (err) {
    if (err) {
      return console.error(err);
    }

    return console.info("Listening at port " + port);
  });
}