"use strict";

var _ = require("lodash");
var parseSectionPages = require("./parse-section-pages");
var transformSections = require("./transform-sections");

module.exports = function getAllPages(config) {
  if (!config) {
    return console.error("getAllPages - Missing configuration");
  }

  var sections = _.keys(config.paths);
  var pages = [].concat // eslint-disable-line prefer-spread
  .apply([], sections.map(function (sectionName) {
    var section = config.paths[sectionName];

    if (_.isFunction(section.content)) {
      var paths = section.content();

      if (paths.keys) {
        return transformSections(sectionName, section, parseSectionPages(sectionName, section, paths));
      }

      console.warn("getAllPages - Section content did not return a require.context!", section);
    }

    if (_.isFunction(section.index)) {
      var indexPage = section.index();

      return [{
        type: "index",
        fileName: sectionName,
        // Function is an object too - important for title/keyword management.
        file: indexPage,
        layout: indexPage,
        section: section,
        url: "/" + sectionName + "/"
      }];
    }

    if (_.isFunction(section)) {
      var sectionPage = section();

      return [{
        type: "index",
        fileName: sectionName,
        file: sectionPage,
        layout: sectionPage,
        section: section,
        url: sectionName === "/" ? "/" : "/" + sectionName + "/"
      }];
    }

    console.warn("getAllPages - Section content was not a function!", section.content);

    return null;
  }));

  var ret = {};

  _.each(pages, function (o) {
    ret[o.url] = o;
  });

  return ret;
};