"use strict";

var _path = require("path");
var _ = require("lodash");
var parseLayout = require("./parse-layout");
var parseIndexPage = require("./parse-index-page");
var parseUrl = require("./parse-url");

module.exports = function parseSectionPages(sectionName, section, modules) {
  var moduleKeys = modules.keys();
  var ret = _.map(moduleKeys, function (name) {
    var fileName = _.trimStart(name, "./") || "";
    var fileNameWithoutExtension = fileName.split(".").slice(0, -1).join(".");
    var file = modules(name);

    var nearestSectionName = Object.keys(section.paths || {}).filter(function (path) {
      return fileNameWithoutExtension.startsWith(path);
    }).sort()[0] || sectionName;
    var layout = parseLayout(section, nearestSectionName);

    // Render index pages through root
    if (_path.basename(fileNameWithoutExtension).endsWith("index")) {
      return {
        type: "index",
        fileName: fileName,
        file: file,
        layout: parseLayout(section, fileNameWithoutExtension),
        section: section,
        sectionName: nearestSectionName,
        url: fileNameWithoutExtension === "index" && sectionName === "/" ? "/" : joinUrl(sectionName, fileNameWithoutExtension.split("/index").slice(0, -1).join(""))
      };
    }

    return {
      type: "page",
      fileName: fileName,
      file: file,
      layout: layout,
      section: section,
      sectionName: nearestSectionName,
      url: parseUrl(section, nearestSectionName, fileNameWithoutExtension)
    };
  });

  // Check for index functions within nested sections
  var checkedSections = {};
  var indexPages = _.map(Object.keys(section && section.paths || {}), function (childSectionName) {
    var childSection = section && section.paths[childSectionName];
    var indexPage = parseIndexPage(section, childSectionName);

    if (!checkedSections[childSectionName] && indexPage) {
      checkedSections[childSectionName] = true;

      return {
        type: "index",
        fileName: "",
        file: indexPage, // Function is an object too - important for title/keyword management.
        layout: indexPage,
        section: childSection,
        sectionName: childSectionName,
        url: joinUrl(sectionName, childSectionName)
      };
    }

    return null;
  }).filter(function (a) {
    return a;
  }) || [];

  if (_.isFunction(section.index)) {
    var indexPage = section.index();

    ret.push({
      type: "index",
      fileName: "",
      file: indexPage, // Function is an object too - important for title/keyword management.
      layout: indexPage,
      section: section,
      sectionName: sectionName,
      url: sectionName === "/" ? "/" : "/" + sectionName + "/"
    });
  }

  return ret.concat(indexPages);
};

// XXXXX: consume this from somewhere
function joinUrl(a, b) {
  var trimmedA = _.trim(a, "/");
  var trimmedB = _.trim(b, "/");

  return "/" + _.trim(trimmedA + "/" + trimmedB, "/") + "/";
}