"use strict";

var _ = require("lodash");

module.exports = function transformSections(sectionName, section, allPages) {
  if (section.paths) {
    return _.flatMap(_.groupBy(allPages, "sectionName"), function (groupPages, name) {
      // Partition index pages from regular ones as those don't need transforming
      var _partitionPages = partitionPages(groupPages),
          pages = _partitionPages.pages,
          indices = _partitionPages.indices;

      // Transform section itself


      if (sectionName === name && section.transform) {
        return section.transform(pages).concat(indices);
      }

      // Use possible subsection transform
      if (section.paths[name] && section.paths[name].transform) {
        return section.paths[name].transform(pages).concat(indices);
      }

      // No transform, return group pages
      return groupPages;
    });
  }

  if (section.transform) {
    var _partitionPages2 = partitionPages(allPages),
        pages = _partitionPages2.pages,
        indices = _partitionPages2.indices;

    return section.transform(pages).concat(indices);
  }

  return allPages;
};

function partitionPages(allPages) {
  // Partition index pages from regular ones as those don't need transforming
  var ret = _.partition(allPages, function (_ref) {
    var type = _ref.type;
    return type === "page";
  });

  return {
    pages: ret[0],
    indices: ret[1]
  };
}